==> radius-1.0.0.tar.gz: radius1.0.0/util/NewData.cxx <==

/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const NewData_cxx_Version = 
"$Id: NewData.cxx,v 1.7 2000/08/10 11:16:55 bogawa Exp $";
   

//Authors: Sunitha Kumar, Cullen Jennings

#include <cstdio>
#include <ctype.h>

#include "cpLog.h"
#include "NewData.hxx"
#include "DataException.hxx"
#include <hash_map>


                              

NewData::NewData():
  hash_val(0),
  hash_valid(false)
{
  const char* str = "";
  buflen = boundary(strlen(str) + 1);
  buf = new char[buflen];
  strcpy(buf, str);
}


NewData::NewData( const char* str, int length ):
  hash_val(0),
  hash_valid(false)
{
   str = str ? str : "";
   buflen = boundary(strlen(str) + 1);
   buf = new char[buflen];
   strcpy(buf, str);
}


NewData::NewData( const char* str ):
  hash_val(0),
  hash_valid(false)
{
   str = str ? str : "";
   buflen = boundary(strlen(str) + 1);
   buf = new char[buflen];
   strcpy(buf, str);
}


NewData::NewData( const string& str):
  hash_val(0),
  hash_valid(false)
{
   buflen = boundary(str.length() + 1);
   buf = new char[buflen];   
   strcpy(buf, str.c_str());
}


NewData::NewData( const mstring& mstr):
  hash_val(0),
  hash_valid(false)
{
  buflen = boundary(mstr.length() + 1);
  buf = new char[buflen];
  strcpy(buf, mstr.c_str());
}


NewData::NewData( int value)
    :
  hash_val(0),
  hash_valid(false)
{
    char str[256];
    sprintf(str, "%d", value);
    buflen = boundary(strlen(str) + 1);
    buf = new char[buflen];
    strcpy(buf, str);
}



NewData::NewData( const NewData& data )
{
    buflen = boundary(strlen(data.buf) + 1);
    buf = new char[buflen];
    strcpy(buf, data.buf);
    hash_val = (data.hash_val);
    hash_valid = (data.hash_valid);
}

	
NewData& 
NewData::operator=( const char* str )
{
    int reqlen = strlen(str) + 1;
    if (reqlen < buflen)
    {
	strcpy(buf, str);
    }
    else
    {
        buflen = boundary(reqlen);
        delete [] buf;
        buf = new char[buflen];
        strcpy(buf, str);
    }
    hash_valid = false;
    return (*this);
}


NewData& 
NewData::operator=( const NewData& data )
{
    if(this != &data)
    {
        int reqlen = strlen(data.buf) + 1;
        if (reqlen < buflen)
        {
            strcpy(buf, data.buf);
        }
        else
        {
            buflen = boundary(reqlen);
            delete [] buf;
            buf = new char[buflen];
            strcpy(buf, data.buf);
        }
        hash_valid = false;

    }

    return (*this);
}
	

const char*
NewData::getData() const
{
    return buf;
}


const char*
NewData::getDataBuf() const
{
    return buf;
}


int
NewData::boundary(int len) const
{
    const int step = 64;
    int res = (len/step);
    if (res == 0)
        return step;
    if (res > 0)
    {
        int count = 1;
        do
        {
	    count = count * 2;
	    if (res < count)
	        return (count * step);
       	} while (res >= count);
        return step;
    }
    return step;
}


#if 0	
void NewData::substr( int start, int length, char* deststr ) const
{
    if ( ( start < 0 ) || (length < 1) )
    {
	cpLog(LOG_ERR, "NewData: substr: incorrect start or length parameters");
	throw DataAccessError(OUT_OF_RANGE);
    }
    else 
    {
	char* str = buffer->getBuffer();
	for ( *deststr = *str; *str != '\0'; deststr++, str++, *deststr = *str );
        
	//assign the null char.
	*deststr = '\0';
    }
}
#endif

	
char 
NewData::getChar(int i) const
{
    if (( i < 0 ) || (i > static_cast<int>(strlen(buf)) ) )
    {
	cpLog(LOG_ERR, "NewData:getchar: i is out of range.");
	throw DataException(
                        "i is out of range",
                        __FILE__,
                        __LINE__
                        );
	return '\0';
    }

    return buf[i];
}

	
void NewData::setchar(int i, char c)
{
    hash_valid = false;

    if (( i < 0) || (i > static_cast<int>(strlen(buf)) ) )
    {
	cpLog(LOG_ERR, "NewData:setchar: i is out of range.");
        throw DataException(
                        "i is out of range",
                        __FILE__,
                        __LINE__
                        );

    }
    else
    {
      buf[i] = c;
    }
}


char 
NewData::operator[](int i)
{
    char ch;
    ch = buf[i];
    return ch;
}


int 
NewData::length() const 
{
  return strlen(buf);
}

	
bool NewData::operator==(const char* str) const
{
    int comparison = strcmp(buf, str);
    return ( comparison == 0 );
}

	
bool NewData::operator==( const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
#if 1
    return ( comparison == 0 );
#else
    if(data.hashfn() == hashfn())
    {
	return ( comparison == 0 );
    }
    else
    {
	return false;
    }
#endif
}

	
bool NewData::operator!=(const char* str) const
{
    int comparison = strcmp(buf, str);
    return ( comparison != 0 );
}


bool NewData::operator!=(const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
    return (  comparison != 0 );

#if 0
    if(data.hashfn() == hashfn())
    {
	return (  comparison != 0 );
    }
    else
    {
	return true;
    }
#endif
}

	
bool
 NewData::operator>(const NewData& data) const 
{
    int comparison = strcmp(buf, data.buf);
    return ( comparison > 0 );
}

	
bool
 NewData::operator<(const NewData& data) const
{
   int comparison = strcmp(buf, data.buf);
   return ( comparison < 0 );
}

	
NewData 
NewData::operator+(const NewData& data) const
{
    char newbuf[strlen(buf) + strlen(data.buf) + 1];
    strcpy(newbuf, buf);
    strcat(newbuf, data.buf);
    NewData temp(newbuf);
    return temp;
}
	
	
NewData 
NewData::operator+(const char* str) const
{
    char newbuf[strlen(buf) + strlen(str) + 1];
    strcpy(newbuf, buf);
    strcat(newbuf, str);
    NewData temp(newbuf);
    return temp;
}
	
/*
void
Data::replace(int startpos, int replaceLen, const char* replaceStr)
{
    char* newbuf = new char[strlen(buf) + 1];
    string str = string(buf);
    str.replace(startpos, replaceLen, replaceStr);
    strcpy (newbuf, str.c_str());
    delete [] buf;
    buf = newbuf;
}
*/
    
void
NewData::replace(int startpos, int endpos, const char* replaceStr)
{
    string str = string(buf);
    str.replace(startpos, endpos, replaceStr);
    int reqlen = str.length() + 1;
    if (reqlen < buflen)
    {
	strcpy(buf, str.c_str());
    }
    else
    {
	buflen = boundary(reqlen);
        char* newbuf = new char[buflen];
        strcpy (newbuf, str.c_str());
        delete [] buf;
        buf = newbuf;
    }
}


void 
NewData::operator+=(const NewData& data)
{
    hash_valid = false;
    *this = *this + data;
}

	
void 
NewData::operator+=(const char* str)
{
    hash_valid = false;
    *this = *this + str;
}
	
        
NewData::~NewData()
{
    delete [] buf;
}


void
NewData::erase()
{
    strcpy (buf, "");
    hash_valid = false;
}



NewData::operator string() const
{
  return string(buf);
}


NewData::operator const char*() const
{
  return buf;
}


NewData::operator mstring() const
{
  return mstring(buf);
}


NewData::operator int() const
{
    return atoi( buf );
}


size_t
NewData::hashfn() 
{
    if(!hash_valid)
    {
	hash<const char*> x;
	hash_val = x(buf);
	hash_valid = true;
    }
    return hash_val;
}


int  NewData::match( NewData match, NewData* retModifiedNewData, bool doReplace, NewData replaceWith)
{
#if 1
    int retVal;
    string::size_type pos = (string(buf)).find(match.getData());
    
    if (pos == string::npos)
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        return NOT_FOUND;
    }

    string::size_type replacePos = pos + match.length();
    retVal = FIRST;

    if (retModifiedNewData)
    {
        (*retModifiedNewData) = (string(buf)).substr(0, pos);
	if(retModifiedNewData->length()) retVal = FOUND;
    }

    if (doReplace)
    {
        if (replacePos <= strlen(buf) )
        {
	    hash_valid = false;
            replace(0, replacePos, replaceWith.getData());
        }
        else
        {
             printf("buf =<%s> match =<%s>\n", buf , match.getData() );
             printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n", 
                    pos,
                    match.length(),
                    replacePos, 
                    strlen(buf) );

        }
    }

    return retVal;
#else

    int pos = static_cast<int>((string(buf)).find(match.getData()));

    if (pos == static_cast<int>(string::npos))
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        pos = NOT_FOUND;
    }
    else
    {
       if (data)
       {
           (*data) = (string(buf)).substr(0, pos);
       }
       if (doReplace)
       {
	   hash_valid = false;
           replace(0, (pos+match.length()), replaceWith.getData());
       }
       else
       {

       }
       if ( (data->length() == 0) && (pos != ( (int) string::npos) ) )
       {
            //the match string is the first item.
            pos = -2;
            pos = FIRST;
       }

    }
    if ( (pos != NOT_FOUND) && (pos != FIRST) )
    {
    pos = FOUND;
    }
    return pos;
#endif
}


/*

int  Data::match( const Data& match, Data* retModifiedData, bool doReplace, const Data& replaceWith)
{
    int retVal;
    string::size_type pos = buf.find(match.getData());
    
    if (pos == string::npos)
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        return NOT_FOUND;
    }

    string::size_type replacePos = pos + match.length();
    retVal = FIRST;

    if (retModifiedData)
    {
        (*retModifiedData) = buf.substr(0, pos);
	if(retModifiedData->length()) retVal = FOUND;
    }

    if (doReplace)
    {
        newbuf = new char[strlen(buf) + 1];
        strcpy(newbuf, replaceWith.buf);
        strcat(newbuf, buf);
        strcpy(
        buf.replace(0, replacePos, replaceWith.getData());
    }

    return retVal;
}

*/

bool isEqualNoCase( const NewData& leftData, const NewData& rightData ) 
{
    string leftbuf(leftData.buf);
    string rightbuf(rightData.buf);
    string::const_iterator leftIter = leftbuf.begin();
    string::const_iterator rightIter = rightbuf.begin();

    while( (leftIter != leftbuf.end()) && (rightIter != rightbuf.end()) )
    {
	if(toupper(*leftIter) != toupper(*rightIter))
	{
	    return false;
	}
	++leftIter;
	++rightIter;
    }

    if( (leftIter != leftbuf.end()) || (rightIter != rightbuf.end()) )
    {
	// since both aren't the same length, they're not equal
	return false;
    }

    return true;
}
    
    
    
void NewData::removeSpaces()
{
    hash_valid = false;
    //removes spaces before and after the characters.
    //Leaves the embedded spaces as is.
    
    if (strlen(buf) == 0)
    {
	return;
    }
    char space = SPACE[0]; 
    char ch = buf[0];
    do
    {
	if (strlen(buf) == 0)
	    break;
        if ( ch == space)
        {
    
            replace(0, strlen(SPACE), "");
	    ch = buf[0];
  	}
        else
	{
	    break;
        }
    }
    while (ch == space);
    

    ch = buf[strlen(buf) - 1];
    do
    {
         
	if (strlen(buf) == 0)
	    break;
        //if there are chars after val, discard .
        if (ch == space)
        {
            replace(strlen(buf) - 1, (strlen(buf) - 1) + strlen(SPACE) , "");
	    ch = buf[strlen(buf) - 1];
	}
	else
	{
	    break;
	}
    }
    while (ch == space);

}
    
    
void
NewData::expand(NewData startFrom, NewData findstr, NewData replstr, NewData delimiter)
{
    hash_valid = false;

    string::size_type startPos = (string(buf)).find(startFrom.getData());
    if (startPos < string::npos)
    {
	string::size_type delimPos  = (string(buf)).find(delimiter.getData(), startPos);
	string::size_type findPos = (string(buf)).find( findstr.getData(), startPos);
    
	while (findPos < delimPos)
	{
	    //found replstr, replace
	    replace( findPos, strlen(findstr.getData()), replstr.getData());
	    //find next.
            //delimPos = buf.find( delimiter.getData(), findPos);
                delimPos = (string(buf)).find( delimiter.getData(), findPos + static_cast<string>(replstr.getData()).size() ); 
	    findPos = (string(buf)).find( findstr.getData(), findPos);
	}
    }
}


void
NewData::deepCopy(const NewData &src, char ** bufPtr, int *bufLenPtr)
{
    *this = src;
}

==> radius-1.1.0.tar.gz: radius1.1.0/util/NewData.cxx <==

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const NewData_cxx_Version =
    "$Id: NewData.cxx,v 1.9 2000/09/20 08:01:34 cvs Exp $";


//Authors: Sunitha Kumar, Cullen Jennings

#include <cstdio>
#include <ctype.h>

#include "cpLog.h"
#include "NewData.hxx"
#include "DataException.hxx"
#include <hash_map>




NewData::NewData():
        hash_val(0),
        hash_valid(false)
{
    const char* str = "";
    buflen = boundary(strlen(str) + 1);
    buf = new char[buflen];
    strcpy(buf, str);
}


NewData::NewData( const char* str, int length ):
        hash_val(0),
        hash_valid(false)
{
    str = str ? str : "";
    buflen = boundary(strlen(str) + 1);
    buf = new char[buflen];
    strcpy(buf, str);
}


NewData::NewData( const char* str ):
        hash_val(0),
        hash_valid(false)
{
    str = str ? str : "";
    buflen = boundary(strlen(str) + 1);
    buf = new char[buflen];
    strcpy(buf, str);
}


NewData::NewData( const string& str):
        hash_val(0),
        hash_valid(false)
{
    buflen = boundary(str.length() + 1);
    buf = new char[buflen];
    strcpy(buf, str.c_str());
}


NewData::NewData( const mstring& mstr):
        hash_val(0),
        hash_valid(false)
{
    buflen = boundary(mstr.length() + 1);
    buf = new char[buflen];
    strcpy(buf, mstr.c_str());
}


NewData::NewData( int value)
        :
        hash_val(0),
        hash_valid(false)
{
    char str[256];
    sprintf(str, "%d", value);
    buflen = boundary(strlen(str) + 1);
    buf = new char[buflen];
    strcpy(buf, str);
}



NewData::NewData( const NewData& data )
{
    buflen = boundary(strlen(data.buf) + 1);
    buf = new char[buflen];
    strcpy(buf, data.buf);
    hash_val = (data.hash_val);
    hash_valid = (data.hash_valid);
}


NewData&
NewData::operator=( const char* str )
{
    int reqlen = strlen(str) + 1;
    if (reqlen < buflen)
    {
        strcpy(buf, str);
    }
    else
    {
        buflen = boundary(reqlen);
        delete [] buf;
        buf = new char[buflen];
        strcpy(buf, str);
    }
    hash_valid = false;
    return (*this);
}


NewData&
NewData::operator=( const NewData& data )
{
    if (this != &data)
    {
        int reqlen = strlen(data.buf) + 1;
        if (reqlen < buflen)
        {
            strcpy(buf, data.buf);
        }
        else
        {
            buflen = boundary(reqlen);
            delete [] buf;
            buf = new char[buflen];
            strcpy(buf, data.buf);
        }
        hash_valid = false;

    }

    return (*this);
}


const char*
NewData::getData() const
{
    return buf;
}


const char*
NewData::getDataBuf() const
{
    return buf;
}


int
NewData::boundary(int len) const
{
    const int step = 64;
    int res = (len / step);
    if (res == 0)
        return step;
    if (res > 0)
    {
        int count = 1;
        do
        {
            count = count * 2;
            if (res < count)
                return (count * step);
        }
        while (res >= count);
        return step;
    }
    return step;
}


#if 0
void NewData::substr( int start, int length, char* deststr ) const
{
    if ( ( start < 0 ) || (length < 1) )
    {
        cpLog(LOG_ERR, "NewData: substr: incorrect start or length parameters");
        throw DataAccessError(OUT_OF_RANGE);
    }
    else
    {
        char* str = buffer->getBuffer();
        for ( *deststr = *str; *str != '\0'; deststr++, str++, *deststr = *str );

        //assign the null char.
        *deststr = '\0';
    }
}
#endif


char
NewData::getChar(int i) const
{
    if (( i < 0 ) || (i > static_cast < int > (strlen(buf)) ) )
    {
        cpLog(LOG_ERR, "NewData:getchar: i is out of range.");
        throw DataException(
            "i is out of range",
            __FILE__,
            __LINE__
        );
        return '\0';
    }

    return buf[i];
}


void NewData::setchar(int i, char c)
{
    hash_valid = false;

    if (( i < 0) || (i > static_cast < int > (strlen(buf)) ) )
    {
        cpLog(LOG_ERR, "NewData:setchar: i is out of range.");
        throw DataException(
            "i is out of range",
            __FILE__,
            __LINE__
        );

    }
    else
    {
        buf[i] = c;
    }
}


char
NewData::operator[](int i)
{
    char ch;
    ch = buf[i];
    return ch;
}


int
NewData::length() const
{
    return strlen(buf);
}


bool NewData::operator==(const char* str) const
{
    int comparison = strcmp(buf, str);
    return ( comparison == 0 );
}


bool NewData::operator==( const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
#if 1 
    return ( comparison == 0 );
#else
    if (data.hashfn() == hashfn())
    {
        return ( comparison == 0 );
    }
    else
    {
        return false;
    }
#endif
}


bool NewData::operator!=(const char* str) const
{
    int comparison = strcmp(buf, str);
    return ( comparison != 0 );
}


bool NewData::operator!=(const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
    return ( comparison != 0 );

#if 0
    if (data.hashfn() == hashfn())
    {
        return ( comparison != 0 );
    }
    else
    {
        return true;
    }
#endif
}


bool
NewData::operator>(const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
    return ( comparison > 0 );
}


bool
NewData::operator<(const NewData& data) const
{
    int comparison = strcmp(buf, data.buf);
    return ( comparison < 0 );
}


NewData
NewData::operator+(const NewData& data) const
{
    char newbuf[strlen(buf) + strlen(data.buf) + 1];
    strcpy(newbuf, buf);
    strcat(newbuf, data.buf);
    NewData temp(newbuf);
    return temp;
}


NewData
NewData::operator+(const char* str) const
{
    char newbuf[strlen(buf) + strlen(str) + 1];
    strcpy(newbuf, buf);
    strcat(newbuf, str);
    NewData temp(newbuf);
    return temp;
}

/*
void
Data::replace(int startpos, int replaceLen, const char* replaceStr)
{
    char* newbuf = new char[strlen(buf) + 1];
    string str = string(buf);
    str.replace(startpos, replaceLen, replaceStr);
    strcpy (newbuf, str.c_str());
    delete [] buf;
    buf = newbuf;
}
*/

void
NewData::replace(int startpos, int endpos, const char* replaceStr)
{
    string str = string(buf);
    str.replace(startpos, endpos, replaceStr);
    int reqlen = str.length() + 1;
    if (reqlen < buflen)
    {
        strcpy(buf, str.c_str());
    }
    else
    {
        buflen = boundary(reqlen);
        char* newbuf = new char[buflen];
        strcpy (newbuf, str.c_str());
        delete [] buf;
        buf = newbuf;
    }
}


void
NewData::operator+=(const NewData& data)
{
    hash_valid = false;
    *this = *this + data;
}


void
NewData::operator+=(const char* str)
{
    hash_valid = false;
    *this = *this + str;
}


NewData::~NewData()
{
    delete [] buf;
}


void
NewData::erase()
{
    strcpy (buf, "");
    hash_valid = false;
}



NewData::operator string() const
{
    return string(buf);
}


NewData::operator const char*() const
{
    return buf;
}


NewData::operator mstring() const
{
    return mstring(buf);
}


NewData::operator int() const
{
    return atoi( buf );
}


size_t
NewData::hashfn()
{
    if (!hash_valid)
    {
        hash < const char* > x;
        hash_val = x(buf);
        hash_valid = true;
    }
    return hash_val;
}


int NewData::match( NewData match, NewData* retModifiedNewData, bool doReplace, NewData replaceWith)
{
#if 1
    int retVal;
    string::size_type pos = (string(buf)).find(match.getData());

    if (pos == string::npos)
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        return NOT_FOUND;
    }

    string::size_type replacePos = pos + match.length();
    retVal = FIRST;

    if (retModifiedNewData)
    {
        (*retModifiedNewData) = (string(buf)).substr(0, pos);
        if (retModifiedNewData->length()) retVal = FOUND;
    }

    if (doReplace)
    {
        if (replacePos <= strlen(buf) )
        {
            hash_valid = false;
            replace(0, replacePos, replaceWith.getData());
        }
        else
        {
            printf("buf =<%s> match =<%s>\n", buf , match.getData() );
            printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n",
                   pos,
                   match.length(),
                   replacePos,
                   strlen(buf) );

        }
    }

    return retVal;
#else

    int pos = static_cast < int > ((string(buf)).find(match.getData()));

    if (pos == static_cast < int > (string::npos))
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        pos = NOT_FOUND;
    }
    else
    {
        if (data)
        {
            (*data) = (string(buf)).substr(0, pos);
        }
        if (doReplace)
        {
            hash_valid = false;
            replace(0, (pos + match.length()), replaceWith.getData());
        }
        else
        {
        }

        if ( (data->length() == 0) && (pos != ( (int) string::npos) ) )
        {
            //the match string is the first item.
            pos = -2;
            pos = FIRST;
        }

    }
    if ( (pos != NOT_FOUND) && (pos != FIRST) )
    {
        pos = FOUND;
    }
    return pos;
#endif
}


/*
 
int  Data::match( const Data& match, Data* retModifiedData, bool doReplace, const Data& replaceWith)
{
    int retVal;
    string::size_type pos = buf.find(match.getData());
    
    if (pos == string::npos)
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        return NOT_FOUND;
    }
 
    string::size_type replacePos = pos + match.length();
    retVal = FIRST;
 
    if (retModifiedData)
    {
        (*retModifiedData) = buf.substr(0, pos);
	if(retModifiedData->length()) retVal = FOUND;
    }
 
    if (doReplace)
    {
        newbuf = new char[strlen(buf) + 1];
        strcpy(newbuf, replaceWith.buf);
        strcat(newbuf, buf);
        strcpy(
        buf.replace(0, replacePos, replaceWith.getData());
    }
 
    return retVal;
}
 
*/

bool isEqualNoCase( const NewData& leftData, const NewData& rightData )
{
    string leftbuf(leftData.buf);
    string rightbuf(rightData.buf);
    string::const_iterator leftIter = leftbuf.begin();
    string::const_iterator rightIter = rightbuf.begin();

    while ( (leftIter != leftbuf.end()) && (rightIter != rightbuf.end()) )
    {
        if (toupper(*leftIter) != toupper(*rightIter))
        {
            return false;
        }
        ++leftIter;
        ++rightIter;
    }

    if ( (leftIter != leftbuf.end()) || (rightIter != rightbuf.end()) )
    {
        // since both aren't the same length, they're not equal
        return false;
    }

    return true;
}



void NewData::removeSpaces()
{
    hash_valid = false;
    //removes spaces before and after the characters.
    //Leaves the embedded spaces as is.

    if (strlen(buf) == 0)
    {
        return ;
    }
    char space = SPACE[0];
    char ch = buf[0];
    do
    {
        if (strlen(buf) == 0)
            break;
        if ( ch == space)
        {

            replace(0, strlen(SPACE), "");
            ch = buf[0];
        }
        else
        {
            break;
        }
    }
    while (ch == space);


    ch = buf[strlen(buf) - 1];
    do
    {

        if (strlen(buf) == 0)
            break;
        //if there are chars after val, discard .
        if (ch == space)
        {
            replace(strlen(buf) - 1, (strlen(buf) - 1) + strlen(SPACE) , "");
            ch = buf[strlen(buf) - 1];
        }
        else
        {
            break;
        }
    }
    while (ch == space);

}


void
NewData::expand(NewData startFrom, NewData findstr, NewData replstr, NewData delimiter)
{
    hash_valid = false;

    string::size_type startPos = (string(buf)).find(startFrom.getData());
    if (startPos < string::npos)
    {
        string::size_type delimPos = (string(buf)).find(delimiter.getData(), startPos);
        string::size_type findPos = (string(buf)).find( findstr.getData(), startPos);

        while (findPos < delimPos)
        {
            //found replstr, replace
            replace( findPos, strlen(findstr.getData()), replstr.getData());
            //find next.
            //delimPos = buf.find( delimiter.getData(), findPos);
            delimPos = (string(buf)).find( delimiter.getData(), findPos + static_cast < string > (replstr.getData()).size() );
            findPos = (string(buf)).find( findstr.getData(), findPos);
        }
    }
}


void
NewData::deepCopy(const NewData &src, char ** bufPtr, int *bufLenPtr)
{
    *this = src;
}
